import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

/**
 * mServer
 * @author Rune Romundstad
 *
 */
public class mServer extends HttpServlet {
	
	
	/**
	 * doPost is the required function in a HttpServlet class. Either doGet or doPost. doPost handles
	 * the request from the client by reading the ServletInputStream, and writing a response through
	 * ServletOutputStream.
	 */
	public void doPost( HttpServletRequest req, HttpServletResponse res ) throws ServletException, IOException {
		
		String ipAddress = req.getRemoteAddr();
		debug( "A request from: ", ipAddress );
		StringBuffer incoming = new StringBuffer();
		ServletInputStream in = req.getInputStream();
		int servletCh;
		while( ( servletCh = in.read() ) != -1 ) {
			incoming.append( (char)servletCh );
		}
		
		/* the simulated position, at trondheim torg */
		double posX = 569616; 
		double posY = 7034301.3125;
		
		/* calls the function for finding the position in the map */
		double[] positions = findPosInMap( posX, posY );
		
		posX = positions[ 0 ];
		posY = positions[ 1 ];
		
		String[] responseFindMap = findMap( posX, posY );
		
		/* stores the result in the String array in three variables */
		String pX = responseFindMap[ 0 ];
		String pY = responseFindMap[ 1 ];
		String mapString = responseFindMap[ 2 ];
		
		/* concatenation of the answer String */
		String sep = ",";
		String answer = pX.concat(sep).concat(pY).concat(sep).concat(mapString);
		
		/* create a outputstream for response to client */
		ServletOutputStream outStream = res.getOutputStream();
		
		byte[] byteStream = answer.getBytes();
		
		/* writing the response to the outputstream */
		for( int i = 0; i < byteStream.length ; i++ ) {
			outStream.write( byteStream[ i ] );
		}
		
		res.setContentType("text/html");
		
	}
	
	/**
	 * findPosInMap is the function for finding the users position in the large map
	 * @param posX is the users position in latitude and Euref89
	 * @param posY is the users position in longitude and Euref89
	 * @return a double array with an X and Y value for the users position in pixels for the large map. The 
	 * values are the number of pixels from the top left corner of the map, in X and Y direction.
	 */
		
	public double[] findPosInMap( double posX, double posY ) {
		
		/* the information from using the georeferencetool. The X and Y value of the top left
		 * corner of the large map, and the scale of the map. One pixel in the map is 2.15 meters in
		 * real life.
		 */
		double topLeftX = 568717.039268647;
		double topLeftY = 7034859.9229171;
		double mapScale = 2.1502181591523;
		
		/* calculates the position in pixels in the map. Take the difference in meters between position
		 * and top left corner and devide by the map scale.
		 */
		double positionX = ( posX - topLeftX ) / mapScale;
		double positionY = ( topLeftY - posY ) / mapScale;
		
		/* stores the position in a double array */
		double[] pos = new double[2];
		pos[0] = positionX; 
		pos[1] = positionY;
		
		return pos;
		
	}
	
	/**
	 * findMap is the function for find the right piece of the large map, and calculate
	 * the users position in the found piece.
	 * @param posX is the users X position in the large map, value returned from findPosInMap function.
	 * @param posY is the users Y position in the large map, value returned from findPosInMap function.
	 * @return a string array with: name of the right map piece together with an X and Y position in pixels in
	 * that specific piece.  
	 */
	
	public String[] findMap( double posX, double posY ) {
		
		String imageFile = null;
		String[] response = new String[3];
		double pixelX = posX;
		double pixelY = posY;
		int resX, resY;
		
		/* Finding the right part of the map by first checking the X-value and then Y.
		 * When finding the right piece, the String imageFile is set to the name of that piece. 
		 * After it have found the right piece, it also calcualates a new X and Y value, within the
		 * piece. */
		if( ( pixelX < 200 ) ) {
			if( ( pixelY < 200 ) ) {
				imageFile ="/trondheim_midtbyen_0_0.png";
			}else if( ( pixelY > 200 && pixelY < 400 ) ) {
				imageFile ="/trondheim_midtbyen_0_200.png";
				pixelY = pixelY - 200;
			}else if( ( pixelY > 400 && pixelY < 600 ) ) {
				imageFile ="/trondheim_midtbyen_0_400.png";
				pixelY = pixelY - 400;
			}
		}else if( ( pixelX > 200 && pixelX < 400 ) ) {
			if( ( pixelY < 200 ) ) {
				imageFile ="/trondheim_midtbyen_200_0.png";
				pixelX = pixelX - 200;
			}else if( ( pixelY > 200 && pixelY < 400 ) ) {
				imageFile ="/trondheim_midtbyen_200_200.png";
				pixelX = pixelX - 200;
				pixelY = pixelY - 200;
			}else if( ( pixelY > 400 ) ) {
				imageFile ="/trondheim_midtbyen_200_400.png";
				pixelX = pixelX - 200;
				pixelY = pixelY - 400;
			}
		}else if( ( pixelX > 400 && pixelX < 600 ) ) {
			if( ( pixelY < 200 ) ) {
				imageFile ="/trondheim_midtbyen_400_0.png";
				pixelX = pixelX - 400;
			}else if( ( pixelY > 200 && pixelY < 400 ) ) {
				imageFile ="/trondheim_midtbyen_400_200.png";
				pixelX = pixelX - 400;
				pixelY = pixelY - 200;
			}else if( ( pixelY > 400 ) ) {
				imageFile ="/trondheim_midtbyen_400_400.png";
				pixelX = pixelX - 400;
				pixelY = pixelY - 400;
			}
		}else if( ( pixelX > 600 ) ) {
			if( ( pixelY < 200 ) ) {
				imageFile ="/trondheim_midtbyen_600_0.png";
				pixelX = pixelX - 600;
			}else if( ( pixelY > 200 && pixelY < 400 ) ) {
				imageFile ="/trondheim_midtbyen_600_200.png";
				pixelX = pixelX - 600;
				pixelY = pixelY - 200;
			}else if( ( pixelY > 400 ) ) {
				imageFile ="/trondheim_midtbyen_600_400.png";
				pixelX = pixelX - 600;
				pixelY = pixelY - 400;
			}
		}
		/* typecasting the position values to int, since the draw method on the client
		 * can't handle pixelvalues of type double. The positions may be somewhat inaccurate
		 * when typecasting.
		 */
		resX = (int)pixelX;
		resY = (int)pixelY;
		
		/*
		 * returning the values as a String array, since one of them are plain text.
		 */
		response[ 0 ] = Integer.toString( resX );
		response[ 1 ] = Integer.toString( resY );
		response[ 2 ] = imageFile;
		
		return response;
	}
	
	/**
	 * debug is used for writing out messages to the console, by taking to String parameters and displays
	 * them to the console.
	 * @param text	is the explaining text which shall be written out.
	 * @param message is the value the text variable explains.
	 */
	public void debug( String text, String message ) {
		System.out.println( text + message );
	}

	
}

