import javax.microedition.lcdui.Canvas;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Graphics;
import javax.microedition.lcdui.Image;

/**
 * MyCanvas
 * @author Rune Romundstad
 *
 */

public class MyCanvas extends Canvas implements CommandListener {
	
	/* declared priavte variables */
	private mobileClient midlet;
	private Image img;
	private Command cmdExit, cmdLocate, cmdOpenBrowser;
	private boolean clearDisplay = true;
	private boolean showFacilities = false;
	private String bottomText = "";
	private String bottomText2 = "Press Locate to see position in map...";
	private int coordX, coordY;

	/* decleared public variables */
	public String imageString = null;
	public String link = null;
	public int[] apartmentsX = { 80, 79 }; 
	public int[] apartmentsY = { 131, 86 };
	public int imgX, imgY;
	
	/**
	 * constructor makes sure it's only one instance of the midlet object. It also creates and add
	 * the commands used.
	 * @param midlet is a object of mobileClient.
	 */
	public MyCanvas( mobileClient midlet ) {
		
		this.midlet = midlet;
		imageString = "/startmap.png";
		
		cmdExit = new Command("Exit", Command.EXIT, 1 );
		cmdLocate = new Command("Locate", Command.OK, 1 );
		cmdOpenBrowser = new Command("Prospect", Command.SCREEN, 1 );
		addCommand( cmdExit );
		addCommand( cmdLocate );
		setCommandListener( this );
			
	}
	
	 /**
	  * paint takes care of all the drawing to screen. Both the images/maps and the drawing on the map.
	  *  
	  */
	public void paint( Graphics g ) {
		
		/* create the map image */
		try {
			img = Image.createImage( imageString );
		}catch( Exception e ) {
			throw new RuntimeException("Unable to load image: " + e);			
		}
		
		/* find the position of the map in the screen */
		int width = getWidth();
		int height = getHeight();
		imgX = ( width - img.getWidth() ) / 2;
		imgY = ( height - img.getWidth() ) / 2;
		int myPosScale = 5;	// size of myPos triangle
		int scale = 5;		// size scale of the other triangles
		
		/* clears the display before it draws. Must be set to true before every repaint call,
		 * if not it draws on top of the previous drawings.
		 */
		if( clearDisplay ) {
			g.setColor( 255, 255, 255 );
			g.fillRect( 0, 0, width, height );
			
			clearDisplay = false;
		}
		
		g.setColor( 0 );
		g.drawString("Mobile Guide v0.7", 10, 0, g.TOP | g.LEFT );
		g.drawString( bottomText, 10, height - 20, g.BOTTOM | g.LEFT );
		g.drawString( bottomText2, 10, height - 10, g.BOTTOM | g.LEFT );
		g.drawImage( img, width/2, height/2, g.VCENTER | g.HCENTER );
		
		/* gets the position from the midlet */
		int myPositionX = Integer.parseInt( midlet.myPosX );
		int myPositionY = Integer.parseInt( midlet.myPosY );
		
		/* If the boolean 'showFacilities' is set to true, it draws all the positions in the map.
		 * This boolean has been implemented because there is not all the screens we want the positions
		 * to be draw. For example in the overview map at the start.
		 */
		if( showFacilities ) {
			/* Draws the clients position in the map as a star */
			g.setColor( 0, 0, 0 );
			g.fillTriangle( ( imgX + myPositionX ) - myPosScale , ( imgY + myPositionY ) + myPosScale , 
					( imgX + myPositionX ),( imgY + myPositionY ) - myPosScale, 
					( imgX + myPositionX ) + myPosScale, ( imgY + myPositionY ) + myPosScale );
			g.fillTriangle( ( imgX + myPositionX ) - myPosScale , ( imgY + myPositionY ) , 
					( imgX + myPositionX ),( imgY + myPositionY ) + ( myPosScale + 3 ), 
					( imgX + myPositionX ) + myPosScale, ( imgY + myPositionY ) );
			
			/* draws all the apartments */
			for( int i = 0; i < apartmentsX.length; i++ ) {
				g.fillTriangle( ( imgX + apartmentsX[ i ]) - scale, ( imgY + apartmentsY[ i ] ) + scale ,
					( imgX + apartmentsX[ i ] ), ( imgY + apartmentsY[ i ] ) - scale , 
					( imgX + apartmentsX[ i ] ) + scale, ( imgY + apartmentsY[ i ] ) + scale );
			}
			/* checks the array stored from the choice menu */
			for( int i = 0; i < midlet.msg.length; i++ ) {
				/* Draws parkingspace in the map, if checked */
				if( midlet.msg[ 0 ] != null && midlet.msg[0].equals( "Parking" ) ){
					g.setColor( 0, 0, 255 );
					g.fillTriangle( ( imgX + 100 ) - scale , ( imgY + 100 ) + scale , 
							( imgX + 100 ),( imgY + 100 ) - scale, 
							( imgX + 100 ) + scale, ( imgY + 100 ) + scale );
					
				}
				/* Draws schools in the map, if checked */
				if( midlet.msg[ 1 ] != null && midlet.msg[ 1 ].equalsIgnoreCase( "schools" ) ){
					g.setColor( 0, 255, 0 );
					g.fillTriangle( ( imgX + 50 ) - scale , ( imgY + 50 ) + scale , 
							( imgX + 50 ),( imgY + 50 ) - scale, 
							( imgX + 50 ) + scale, ( imgY + 50 ) + scale );
					
				}
				/* Draws kindergartens in the map, if checked */
				if( midlet.msg[ 2 ] != null && midlet.msg[ 2 ].equalsIgnoreCase( "kindergartens" ) ){
					g.setColor( 255, 0, 0 );
					g.fillTriangle( ( imgX + 140 ) - scale , ( imgY + 180 ) + scale , 
							( imgX + 140 ),( imgY + 180 ) - scale, 
							( imgX + 140 ) + scale, ( imgY + 180 ) + scale );
					
				}
			}
		}
		
	}
	/**
	 * commandAction listen as in the other class on the commands. If 'exit' is pressed it calls the
	 * exitMidlet in the mobileClient class. If 'locate' is pressed it clears the display, sets the
	 * showFacilities true, sets the imageString and the explanation text and calls repaint to draw it
	 * all.
	 * If 'openBrowser' is pressed it calls the openBrowser function with the link set below in the
	 * pointerPressed function, as parameter.
	 */
	public void commandAction( Command c, Displayable d ) {
		
		if( c == cmdExit ) {
			midlet.exitMidlet();
		}else if( c == cmdLocate ) {
			clearDisplay = true;
			showFacilities = true;
			imageString = midlet.mapString;
			
			//bottomText = midlet.result;
			bottomText = "black = apartment, blue = parking";
			bottomText2 = "green = school, red = kindergarten";
			
			repaint();
			removeCommand( cmdLocate );
			addCommand( cmdOpenBrowser );
		}else if( c == cmdOpenBrowser ) {
			midlet.openBrowser( link );
		}
	}
	
	/**
	 * pointerPressed listens if something is pressed at the screen. And it gets where in the screen
	 * it have been pressed through the x and y inparameters. The functions then checks if the user
	 * have pressed at one of the apartments, and if it displays short information about the apartment
	 * below the map, and sets the link variable to the apartments prospect.
	 */
	public void pointerPressed( int x, int y ) {
		coordX = x;
		coordY = y;
		clearDisplay = true;
		if( coordX > ( ( imgX + apartmentsX[ 0 ] ) - 10 ) && coordX < ( ( imgX + apartmentsX[ 0 ] ) + 10 ) ) {
			if( coordY > ( ( imgY + apartmentsY[ 0 ] ) - 10 ) && coordY < ( ( imgY + apartmentsY[ 0 ] ) + 10 ) ) {
				bottomText = "Erling Skakkesgt, 3 rooms";
				bottomText2 = "60 square metre, 1.500.000kr";
				link = "http://webhotell.tinde.no/dnbnor/index.asp?pId=212500&offset=4";
				repaint();
			}
			
		}if( coordX > ( ( imgX + apartmentsX[ 1 ] ) - 10 ) && coordX < ( ( imgX + apartmentsX[ 1 ] ) + 10 ) ) {
			if( coordY > ( ( imgY + apartmentsY[ 1 ] ) - 10 ) && coordY < ( ( imgY + apartmentsY[ 1 ] ) + 10 ) ) {
				bottomText = "Presidentveita, 4 rooms";
				bottomText2 = "120 square metre, 3.200.000";
				link = "http://webhotell.tinde.no/dnbnor/index.asp?pId=212954&offset=5";
				repaint();
			}
			
		}else {
			repaint();
		}
		
		
		
	}


}
