import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;
import javax.microedition.io.*;

import java.io.*;

/**
 * mobileClient
 * @author Rune Romundstad
 *
 */

public class mobileClient extends MIDlet implements CommandListener {
	
	/* declearing variables */
	private static String serverURL = "http://85.221.61.12/servlet/mServer"; // address to the server
	private static String message = "message";
	private MyCanvas canvas;
	private Display display;
	private ChoiceGroup choiceTypes;
	private int choiceIndex;
	private Command exit, confirm;
	private Form choiceForm;
	
	public String mapString = null;
	public String[] msg = null;
	public String result;
	public Image image;
	public String[] choicesMade = null;
	public String myPosX, myPosY;
	
	/**
	 * Constructor
	 * Sets up the choice form and the Command buttons.
	 */
	public mobileClient() {
		
		display = Display.getDisplay( this );
		canvas = new MyCanvas( this );	// creates an instance of the MyCanvas class.
		
		/* Creates the selection form where user selects facilities. */
		choiceTypes = new ChoiceGroup( "Select facilities to show in map: " , Choice.MULTIPLE );
		choiceTypes.append( "Parking", null );
		choiceTypes.append( "Schools", null );
		choiceTypes.append( "Kindergartens", null );
		
		exit = new Command( "Exit", Command.EXIT, 1 );			// creates the exit command
		confirm = new Command( "Confirm", Command.SCREEN, 1 );	// creates the confirm command
		choiceForm = new Form( "ChoiceTypes" );					
		choiceIndex = choiceForm.append( choiceTypes );
		choiceForm.addCommand( exit );							// adds exit to the form
		choiceForm.addCommand( confirm );						// adds confirm to the form
		choiceForm.setCommandListener( this );					// sets a commandlistener on the commands
		
	}
	
	/**
	 * startApp is a required function for a MIDlet.
	 * It takes the application out of the pause state and makes the connection
	 * to the server.
	 */
	public void startApp() {
		
		/* running the connectServer and makes the communication. In return it gets a
		 * string stored in result.
		 */
		result = connectServer( message, serverURL );
		//System.out.println("The result from the server was: " + result);
		
		/* Splitting up the result String into three variables. Xpos, Ypos and mapString. */
		int index = result.indexOf(',');
		int lastIndex = result.lastIndexOf(',');
		myPosX = result.substring( 0 , index );
		myPosY = result.substring( index + 1, lastIndex );
		mapString = result.substring( lastIndex + 1 );
		System.out.println("Result from server was : " + myPosX + " " + myPosY +
				" " + mapString );
		
		/* sets display to the choice form */
		display.setCurrent( choiceForm );
		
	}
	/**
	 * pauseApp is a required function to define. It takes care of what that should happend when 
	 * the application is for example waiting to start.
	 */
	public void pauseApp() {
		
	}
	
	/**
	 * destroyApp is also a required function. It is used to quit the application.
	 * @param unconditional
	 */
	public void destroyApp( boolean unconditional ) {
		
	}
	
	/**
	 * exitMidlet created to make a controlled closure for the application. It calls the destroyApp
	 * function with the boolean value 'unconditional' set to true. It also calls the function
	 * 'notifyDestroyed' to tell the system it is finished.
	 *
	 */
	public void exitMidlet() {
		destroyApp( true );
		notifyDestroyed();
	}
	
	/**
	 * commandAction listen on the command buttons. If pressing 'confirm' it reads the choices
	 * made in the menu and stores it for later use. Or if pressing 'exit' it calls the exitMidlet
	 * function.
	 */
	public void commandAction( Command command, Displayable displayable ) {
		
		// Gets the choices made in the startup form.
		if( command == confirm ) {
			boolean picks[] = new boolean[ choiceTypes.size() ];
			msg = new String[ choiceTypes.size() ];
			choiceTypes.getSelectedFlags( picks );
			
			for( int i = 0; i < picks.length; i++ ) {
				if( picks[ i ] ) {
					msg[ i ] = new String( choiceTypes.getString( i ) );
				}
			}
			
			for( int i = 0; i < msg.length; i++ ) {
				if( msg [ i ] != null )
					System.out.println( msg[ i ] );
				
			}
			
			display.setCurrent( getCanvas() );
			choiceForm.delete( choiceIndex );
			choiceForm.removeCommand( confirm );
			
		} else if( command == exit ) {
			exitMidlet();
			
		}
	}
	
	/**
	 * getCanvas return a canvas. If no canvas exist, it creates a new.
	 * @return a instance of the Object MyCanvas.
	 */
	MyCanvas getCanvas() {
		
		if( canvas == null ) {
			canvas = new MyCanvas( this );
			
		}
		return canvas;
	}
	
	/**
	 * connectServer makes a connection to server through http, sends request and receive response.
	 * @param reqMessage is the message sent to server
	 * @param url is the server address
	 * @return responseFromServer is the response read from the server
	 */
	
	private String connectServer( String reqMessage, String url ) {
		
		HttpConnection httpCon = null;
		DataInputStream dataInput = null;
		DataOutputStream dataOut = null;
		StringBuffer buf = new StringBuffer();
		String responseFromServer = null;
		
		try {
			/* open connection with server */
			httpCon = ( HttpConnection ) Connector.open( url, Connector.READ_WRITE);
			/* set requestmethod to post, it is GET by default */
			httpCon.setRequestMethod( HttpConnection.POST); 
			/* send the request, byte by byte */
			dataOut = httpCon.openDataOutputStream();
			byte[] request = reqMessage.getBytes();
			for( int i = 0; i < request.length; i++ ) {
				dataOut.writeByte( request[i] );
			}
			
			/* retrieve the response from server */
			dataInput = new DataInputStream( httpCon.openInputStream() );
			int ch;
			
			while( ( ch = dataInput.read() ) != -1 ) {
				buf.append( (char)ch );
			}
			
			responseFromServer = buf.toString();
			
			
		}catch( Exception e ) {
			e.printStackTrace();
			buf = new StringBuffer("ERROR!");
		}finally {
			/* free up streams and connections to help the garbage collector */
			try {
				if( httpCon != null ) httpCon.close();
				if( dataInput != null ) dataInput.close();
				if( dataOut != null ) dataOut.close();
			}catch( IOException ioe ) {
				ioe.printStackTrace();
			}
		}
		
		return responseFromServer;
		
	}
	
	/**
	 * openBrowser gets a urlAddress as parameter, and opens the Internet browser with that url.
	 * @param urlAddress is the Internet address to the specific prospect.
	 */
	public void openBrowser( String urlAddress ) {
		try {
			platformRequest( urlAddress );
			canvas.repaint();
		}catch( ConnectionNotFoundException ce ) {		//ConnectionNotFoundException
			ce.printStackTrace();
		}
		
	}

}